/* options.c

Copyright (C) 2005 Falko Schmidt.

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to
deal in the Software without restriction, including without limitation the
rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
sell copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies of the Software and its documentation and acknowledgment shall be
given in the documentation and software packages that this Software was
used.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

*/

#include <dirent.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <libgen.h>

#include "feh.h"
#include "filelist.h"
#include "options.h"

void fmmode(void)
{
   char *initial_file_path;
   char *initial_file_dir;
   struct dirent *dirstr;
   struct stat statstr;
   DIR *directory;
   int i,j = 0;
   gib_list *remove_list = NULL;

   initial_file_path = (FEH_FILE(filelist->data)->filename);
   initial_file_dir = strdup(initial_file_path);
   initial_file_dir = dirname(initial_file_dir);

   /* in case fmmode is invoked with an image with relative path. */
   if (!strcmp(initial_file_dir,"."))
   {
     /* FIXME shitty GNUism - buggy cmp's */
      initial_file_dir = get_current_dir_name();
      initial_file_path = estrjoin("", initial_file_dir, "/", 
                                   initial_file_path, NULL);
      j = 1;
   }

   if ((directory = opendir(initial_file_dir)) == NULL)
      {
         if (!opt.quiet)
            weprintf("couldn't open directory %s:", initial_file_dir);
         free(initial_file_dir);
         D_RETURN_(5);
      }
	
   chdir(initial_file_dir);
    
   /* add all files in the current directory to the filelist. */
   while ((dirstr = readdir(directory)) != NULL) {
      stat(dirstr->d_name, &statstr);
      if (S_ISDIR(statstr.st_mode))
         ;
      else if (S_ISREG(statstr.st_mode))
      {
         char *newfile;
         newfile = estrjoin("", initial_file_dir, "/", dirstr->d_name, NULL);
         add_file_to_filelist_recursively(newfile, FILELIST_FIRST);
         free(newfile);
         filelist_len++;
      }
   }
   closedir(directory);

   /* sorts the filelist, since opt.sort=SORT_FILENAME is set automatically. */
   feh_prepare_filelist();

   current_file = filelist;
   if (j == 1) {
     remove_list= gib_list_add_front(remove_list, gib_list_last(current_file));
     current_file = filelist;
   }
   while (i<filelist_len &&
          strcmp((FEH_FILE(current_file->data)->filename),initial_file_path))
      {
         filelist = gib_list_add_end(filelist,
                    feh_file_new(FEH_FILE(current_file->data)->filename));
         remove_list = gib_list_add_front(remove_list, current_file);
         current_file = current_file->next;
         i++;
      }
   if (j == 0)
     remove_list = gib_list_add_front(remove_list, current_file);

   if (remove_list)
   {
      for (current_file=remove_list;current_file;
           current_file=current_file->next)
       filelist = gib_list_remove(filelist, (gib_list *) current_file->data);

      gib_list_free(remove_list);
   }

   free(initial_file_dir);
}
